"use client";

import React, { useState, useEffect } from "react";
import { AdvancedMarker, Pin } from "@vis.gl/react-google-maps";
import GoogleMap, { PolylineMap } from "@/components/googleMap";
import { Box } from "@chakra-ui/react";
import { generateColor, getStatusColors } from "@/utils/common";
import { useSession } from "next-auth/react";
import eventSource from "@/utils/eventSource";
import { User } from "../variables/location";
import { usersRequestList } from "@/app/lib/users/action";
import { getIbcRequestList } from "@/app/lib/ibc/action";
import { UserInterface } from "@/types/user/user";
import Filters from "@/components/filters/Filters";
import { MultiSelectOption } from "@/components/fields/MultiSelect";
import { useSearchParams } from "next/navigation";
import { SiGoogleforms } from "react-icons/si";
import { IbcInterface } from "@/types/settings/ibc/ibc";

const Live = () => {
  const [users, setUsers] = useState<User[]>([]);
  const [usersDropdown, setUsersDropdown] = useState<MultiSelectOption[]>([]);
  const [ibcsDropdown, setIbcsDropdown] = useState<MultiSelectOption[]>([]);
  const searchParams = useSearchParams();
  const { data: session } = useSession();

  const ibc_ids = searchParams.get("ibc_ids");
  const user_ids = searchParams.get("user_ids");

  const fetchUsers = async () => {
    const response = await usersRequestList({pagination:'false'});
    if (response.data && response.data.length > 0) {
      setUsersDropdown(
        response.data.map((user: UserInterface) => ({
          label: `${user.first_name} ${user.last_name ?? ""}`,
          value: user.id,
        }))
      );
    }
  };

  const fetchIbcs = async () => {
    const response = await getIbcRequestList({ pagination: "false" });
    if (response?.data && response?.data.length > 0) {
      setIbcsDropdown(
        response?.data.map((ibc: IbcInterface) => ({
          label: `${ibc.area_name}`,
          value: ibc.id,
        }))
      );
    }
  };

  useEffect(() => {
    let customEventSource: eventSource | null = null;
  
    const fetchLiveLocations = async () => {
      if (session?.user?.token) {
        const token = session.user.token;
  
        // Construct the query string
        const params = [];
        if (user_ids) {
          params.push(`user_ids=${user_ids}`);
        }
        if (ibc_ids) {
          params.push(`ibc_ids=${ibc_ids}`);
        }
  
        // Build the URL with query parameters
        const queryString = params.length ? `?${params.join("&")}` : "";
        const url = `${process.env.NEXT_PUBLIC_API_BASE_URL}v1/locations/live${queryString}`;
  
        // Close existing EventSource before opening a new one
        if (customEventSource) {
          customEventSource.close();
        }

        // Clear state to prevent UI from showing stale data
        setUsers([]);
  
        customEventSource = new eventSource(url, token);

        customEventSource.onmessage((data) => {
          console.log("EventSource data:", data);
          const parsedData: User[] = data;

          // Update user locations with parsed latitude and longitude
          setUsers(
            parsedData.map((user) => ({
              ...user,
              locations: user.locations.map((loc) => ({
                ...loc,
                latitude: parseFloat(loc.latitude),
                longitude: parseFloat(loc.longitude),
              })),
            }))
          );
        });
  
        customEventSource.onerror = (error) => {
          console.error("EventSource error:", error);
        };
      }
    };
  
    fetchLiveLocations();
  
    return () => {
      if (customEventSource) {
        customEventSource.close();
      }
    };
  }, [session?.user?.token, user_ids, ibc_ids]);  

  useEffect(() => {
    fetchUsers();
    fetchIbcs();
  }, [session?.user?.token]);

  return (
    <Box my={20}>
      <Box pb={6}>
        <Filters users={usersDropdown} ibcs={ibcsDropdown} showBatches={false} showExport={false} showSearch={false} showDates={false} />
      </Box>
      <Box>
        <GoogleMap
          defaultCenter={{
            lat: users?.[0]?.locations?.[0]?.latitude || 24.9553,
            lng: users?.[0]?.locations?.[0]?.longitude || 67.0586,
          }}
          defaultZoom={15}
        >
          {users.map((user) => {
            const { first_name, last_name, locations } = user;
            const currentPosition = locations[locations.length - 1];
            const { background, borderColor, glyphColor } = getStatusColors(
              currentPosition.status
            );

            return (
              <React.Fragment key={user.id}>
                <AdvancedMarker
                  position={{
                    lat: currentPosition.latitude,
                    lng: currentPosition.longitude,
                  }}
                  clickable={true}
                  onClick={() =>
                    alert(
                      `${first_name} ${last_name ?? ""} is currently ${
                        currentPosition.status
                      }`
                    )
                  }
                  title={`${first_name} ${last_name ?? ""} (${
                    currentPosition.status
                  })`}
                >
                  <Pin
                    background={background}
                    borderColor={borderColor}
                    glyphColor={glyphColor}
                  />
                </AdvancedMarker>
                {locations.map((loc) => {
                  if(loc.status == 'attempting'){
                    return (
                      <AdvancedMarker
                        key={loc.id}
                        position={{
                          lat: loc.latitude,
                          lng: loc.longitude,
                        }}
                        clickable={true}
                        title={`${first_name} ${last_name ?? ""} (${loc.status})`}>
                        <FormMarker />
                      </AdvancedMarker>
                    );
                  }
                  return null;
                })}
                <PolylineMap
                  path={locations.map((loc) => ({
                    lat: loc.latitude,
                    lng: loc.longitude,
                  }))}
                  strokeColor={generateColor(user.id)}
                />
              </React.Fragment>
            );
          })}
        </GoogleMap>
      </Box>
    </Box>
  );
};

const FormMarker = () => {

  return (
    <div
      style={{
        width: "20px", // Adjust width as needed
        height: "20px", // Adjust height as needed
      }}>
      <SiGoogleforms
        style={{
          width: "100%",
          height: "100%",
        }}
      />
    </div>
  );
};

export default Live;
